﻿using ProjectMonorail.Drawnings;
using ProjectMonorail.Exceptions;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace ProjectMonorail.CollectionGenericObjects;

public class ListGenericObjects<T> : ICollectionGenericObjects<T>
    where T : class
{
    /// <summary>
    /// Список объектов, которые храним
    /// </summary>
    private readonly List<T?> _collection;

    /// <summary>
    /// Максимально допустимое число объектов в списке
    /// </summary>
    private int _maxCount;

    public int Count => _collection.Count;

    public int MaxCount
    {
        get => _maxCount;
        set
        {
            if (value > 0)
            {
                _maxCount = value;
            }
        }
    }

    public CollectionType GetCollectionType => CollectionType.List;

    /// <summary>
    /// Конструктор
    /// </summary>
    public ListGenericObjects()
    {
        _collection = new();
    }

    public T? Get(int position)
    {
        if (position < 0 || position >= Count) return null;
        return _collection[position];
    }

    public int Insert(T obj, IEqualityComparer<DrawningLocomotive?>? comparer = null)
    {
        // TODO выброс ошибки, если переполнение
        // TODO выброс ошибки, если такой объект есть в коллекции
        if (Count == _maxCount)
        {
            throw new CollectionOverflowException(Count);
        }

        for (int i = 0; i < Count; i++)
        {
            if (comparer.Equals((_collection[i] as DrawningLocomotive), (obj as DrawningLocomotive)))
            {
                throw new ObjectAlreadyInCollectionException(i);
            }
        }

        _collection.Add(obj);
        return _collection.Count;
    }

    public int Insert(T obj, int position, IEqualityComparer<DrawningLocomotive?>? comparer = null)
    {
        if (position < 0 || position > Count)
        {
            throw new PositionOutOfCollectionException(position);
        }

        if (Count == _maxCount)
        {
            throw new CollectionOverflowException(Count);
        }

        for (int i = 0; i < Count; i++)
        {
            if (comparer.Equals((_collection[i] as DrawningLocomotive), (obj as DrawningLocomotive)))
            {
                throw new ObjectAlreadyInCollectionException(i);
            }
        }

        _collection.Insert(position, obj);
        return position;
    }

    public T? Remove(int position)
    {
        if (position < 0 || position >= Count) throw new PositionOutOfCollectionException(position);
        T? obj = _collection[position];
        _collection.RemoveAt(position);
        return obj;
    }

    public IEnumerable<T?> GetItems()
    {
        for (int i = 0; i < Count; ++i)
        {
            yield return _collection[i];
        }
    }

    public void CollectionSort(IComparer<T?> comparer)
    {
        _collection.Sort(comparer);
    }
}
