﻿using ProjectMonorail.Entities;

namespace ProjectMonorail.Drawnings;

public class DrawningMonorail : DrawningLocomotive
{
    /// <summary>
    /// Инициализация свойств
    /// </summary>
    /// <param name="speed">Скорость</param>
    /// <param name="weight">Вес</param>
    /// <param name="bodyColor">Основной цвет</param>
    /// <param name="additionalColor">Дополнительный цвет</param>
    /// <param name="magneticRail">Признак наличия магнитного рельса</param>
    /// <param name="secondCabin">Признак наличия второй кабины</param>
    public DrawningMonorail(int speed, double weight, Color bodyColor, Color additionalColor, bool magneticRail, bool secondCabin) : base(130,55)
    {
        EntityLocomotive = new EntityMonorail(speed, weight, bodyColor, additionalColor, magneticRail, secondCabin);
    }
    public DrawningMonorail(EntityMonorail locomotive) : base(125, 80)
    {
        EntityLocomotive = new EntityMonorail(locomotive.Speed, locomotive.Weight, locomotive.BodyColor, locomotive.AdditionalColor, locomotive.MagneticRail, locomotive.SecondCabin);
    }

    /// <summary>
    /// Прорисовка объекта
    /// </summary>
    /// <param name="g"></param>
    public override void DrawTransport(Graphics g)
    {
        if (EntityLocomotive == null || EntityLocomotive is not EntityMonorail monorail || !_startPosX.HasValue || !_startPosY.HasValue)
        {
            return;
        }
        Pen pen = new(Color.Black);
        Brush additionalBrush = new SolidBrush(monorail.AdditionalColor);

        //магнитная рельса
        if (monorail.MagneticRail)
        {
            g.FillRectangle(additionalBrush, _startPosX.Value, _startPosY.Value + 48, 130, 5);
        }

        //вторая кабина

        if (monorail.SecondCabin)
        {
            Point[] second_cabin = {
                new Point(_startPosX.Value + 125, _startPosY.Value),
                new Point(_startPosX.Value + 132, _startPosY.Value + 20),
                new Point(_startPosX.Value + 132, _startPosY.Value + 40),
                new Point(_startPosX.Value + 125, _startPosY.Value + 40),
                new Point(_startPosX.Value + 125, _startPosY.Value + 20),
                new Point(_startPosX.Value + 132, _startPosY.Value + 20)
            };
            g.DrawPolygon(pen, second_cabin);
            g.FillPolygon(additionalBrush, second_cabin);

        }
        base.DrawTransport(g);
    }
}
