﻿namespace ProjectMonorail.Entities;
public class EntityMonorail : EntityLocomotive
{
    /// <summary>
    /// Дополнительный цвет (для опциональных элементов)
    /// </summary>
    public Color AdditionalColor { get; private set; }
    public void SetAdditionalColor(Color color)
    {
        AdditionalColor = color;
    }
    /// <summary>
    /// Признак (опция) наличия обвеса
    /// </summary>
    public bool MagneticRail { get; private set; }
    /// <summary>
    /// /// Признак (опция) наличия магнитного рельса
    /// /// </summary>
    public bool SecondCabin { get; private set; }
    /// <summary>
    /// Признак (опция) наличия второй кабины
    /// </summary>
    public double Step => Speed * 100 / Weight;

    public EntityMonorail(int speed, double weight, Color bodyColor, Color additionalColor, bool magneticRail, bool secondCabin) : base(speed, weight, bodyColor)
    {
        AdditionalColor = additionalColor;
        MagneticRail = magneticRail;
        SecondCabin = secondCabin;
    }

    /// <summary>
    /// Получение строк со значениями свойств продвинутого объекта класса-сущности
    /// </summary>
    /// <returns></returns>
    public override string[] GetStringRepresentation()
    {
        return new[] { nameof(EntityMonorail), Speed.ToString(), Weight.ToString(), BodyColor.Name, AdditionalColor.Name,
                        MagneticRail.ToString(), SecondCabin.ToString()};
    }


    /// <summary>
    /// Создание продвинутого  объекта из массива строк
    /// </summary>
    /// <param name="strs"></param>
    /// <returns></returns>
    public static EntityMonorail? CreateEntityMonorail(string[] strs)
    {
        if (strs.Length != 7 || strs[0] != nameof(EntityMonorail))
        {
            return null;
        }
        return new EntityMonorail(Convert.ToInt32(strs[1]), Convert.ToDouble(strs[2]), Color.FromName(strs[3]),
                                    Color.FromName(strs[4]), Convert.ToBoolean(strs[5]), Convert.ToBoolean(strs[6]));
    }
}
