﻿using ProjectMonorail.Drawnings;
using ProjectMonorail.Entities;
using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Forms;

namespace ProjectMonorail;

public partial class FormLocomotiveConfig : Form
{
    private DrawningLocomotive? _locomotive;

    private event LocomotiveDelegate? LocomotiveDelegate;

    public FormLocomotiveConfig()
    {
        InitializeComponent();
        panelRed.MouseDown += Panel_MouseDown;
        panelGreen.MouseDown += Panel_MouseDown;
        panelBlue.MouseDown += Panel_MouseDown;
        panelYellow.MouseDown += Panel_MouseDown;
        panelWhite.MouseDown += Panel_MouseDown;
        panelGray.MouseDown += Panel_MouseDown;
        panelBlack.MouseDown += Panel_MouseDown;
        panelPurple.MouseDown += Panel_MouseDown;

        buttonCancel.Click += (sender, e) => Close();
    }

    public void AddEvent(LocomotiveDelegate locomotiveDelegate)
    {
        LocomotiveDelegate += locomotiveDelegate;
    }
    private void DrawObject()
    {
        Bitmap bmp = new(pictureBoxObject.Width, pictureBoxObject.Height);
        Graphics gr = Graphics.FromImage(bmp);
        _locomotive?.SetPictureSize(pictureBoxObject.Width, pictureBoxObject.Height);
        _locomotive?.SetPosition(5, 5);
        _locomotive?.DrawTransport(gr);
        pictureBoxObject.Image = bmp;
    }

    private void labelObject_MouseDown(object sender, MouseEventArgs e)
    {
        (sender as Label)?.DoDragDrop((sender as Label)?.Name ?? string.Empty, DragDropEffects.Move | DragDropEffects.Copy);
    }

    private void panelObject_DragEnter(object sender, DragEventArgs e)
    {
        e.Effect = e.Data?.GetDataPresent(DataFormats.Text) ?? false ? DragDropEffects.Copy : DragDropEffects.None;
    }

    private void panelObject_DragDrop(object sender, DragEventArgs e)
    {
        switch (e.Data?.GetData(DataFormats.Text)?.ToString())
        {
            case "labelSimpleObject":
                _locomotive = new DrawningLocomotive((int)numericUpDownSpeed.Value, (double)numericUpDownWeight.Value, Color.White);
                break;
            case "labelModifiedObject":
                _locomotive = new DrawningMonorail((int)numericUpDownSpeed.Value, (double)numericUpDownWeight.Value, Color.White,
                Color.Black, checkBoxMagneticRail.Checked, checkBoxSecondCabin.Checked);
                break;
        }
        labelBodyColor.BackColor = Color.Empty;
        labelAdditionalColor.BackColor = Color.Empty;
        DrawObject();
    }

    private void Panel_MouseDown(object? sender, MouseEventArgs e)
    {
        (sender as Control)?.DoDragDrop((sender as Control).BackColor, DragDropEffects.Move | DragDropEffects.Copy);
    }
    //смена цветов
    private void labelBodyColor_DragEnter(object sender, DragEventArgs e)
    {
        if (e.Data?.GetDataPresent(typeof(Color)) ?? false)
        {
            e.Effect = DragDropEffects.Copy;
        }
        else
        {
            e.Effect = DragDropEffects.None;
        }
    }
    private void labelBodyColor_DragDrop(object sender, DragEventArgs e)
    {
        if (_locomotive != null)
        {
            _locomotive.EntityLocomotive.SetBodyColor((Color)e.Data.GetData(typeof(Color)));
            DrawObject();
        }
    }
    private void labelAdditionalColor_DragEnter(object sender, DragEventArgs e)
    {
        if (_locomotive != null && _locomotive is DrawningMonorail)
        {
            if (e.Data?.GetDataPresent(typeof(Color)) ?? false)
            {
                e.Effect = DragDropEffects.Copy;
            }
            else
            {
                e.Effect = DragDropEffects.None;
            }
        }
    }
    private void labelAdditionalColor_DragDrop(object sender, DragEventArgs e)
    {
        if (_locomotive != null && _locomotive.EntityLocomotive is EntityMonorail _monorail)
        {
            if (e.Data.GetDataPresent(typeof(Color)))
            {
                _monorail.SetAdditionalColor((Color)e.Data.GetData(typeof(Color)));

            }
            DrawObject();
        }
    }
    private void ButtonAdd_Click(object sender, EventArgs e)
    {

        if (_locomotive != null)
        {
            LocomotiveDelegate?.Invoke(_locomotive);
            Close();
        }

    }
}
